/*--------------------------------------------------------------------------------------+
|
|     $Source: examples/Gridline/Gridline.cs $
|
|  $Copyright: (c) 2018 Bentley Systems, Incorporated. All rights reserved. $
|
+--------------------------------------------------------------------------------------*/
using Bentley.DgnPlatformNET;
using Bentley.DgnPlatformNET.Elements;
using Bentley.GeometryNET;
using Bentley.Building.Api;

using BBA = Bentley.Building.Api;

namespace GridLineTest
{
    class GridLineRead : DgnPrimitiveTool
    {
        public BBA.ITFLoadableProject GetCurrentProject()
        {
            BBA.ITFApplication appInst = new BBA.TFApplicationList();

            BBA.ITFLoadableProject proj;
            BBA.ITFLoadableProjectList projList;

            if (0 == appInst.GetProject(0, out projList) && projList != null)
            {
                proj = projList.AsTFLoadableProject;
                return proj;
            }

            return null;
        }

        public void ReadGridLine()
        {
            BBA.ITFDrawingGrid drawingGrid = null;
            BBA.ITFGridSystemList gridSystems = null;

            BBA.ITFLoadableProject proj = GetCurrentProject();

            if (null == proj)
                return;

            proj.GetDrawingGrid(false, 0, out drawingGrid);

            if (null == drawingGrid)
                return;

            drawingGrid.GetGridSystems(0, out gridSystems);
            BBA.ITFGridSystemList gridSystemNode = gridSystems;

            for (BBA.ITFGridSystem gridSystem = gridSystemNode.AsTFGridSystem;
                gridSystem != null;
                gridSystemNode.GetNext("", out gridSystemNode), gridSystem = (gridSystemNode != null) ? gridSystemNode.AsTFGridSystem : null)
            {
                BBA.ITFGridCurveList curves;
                gridSystem.GetGridCurves(0, out curves);
                BBA.ITFGridCurveList gridCurveNode = curves;

                for (BBA.ITFGridCurve gridCurve = gridCurveNode.AsTFGridCurve;
                    gridCurve != null;
                    gridCurveNode.GetNext("", out gridCurveNode), gridCurve = gridCurveNode != null ? gridCurveNode.AsTFGridCurve : null)
                {
                    BBA.TFdGridCurveType curveType;
                    gridCurve.GetType(0, out curveType);
                    double gridValue;
                    gridCurve.GetValue(0, out gridValue);
                }
            }
        }

        public GridLineRead(int toolId, int prompt) : base(toolId, prompt)
        {
        }

        protected override void OnRestartTool()
        {
            InstallNewInstance();
        }

        protected override void OnCleanup()
        {
        }

        protected override void OnPostInstall()
        {
            Bentley.DgnPlatformNET.AccuSnap.SnapEnabled = true;
            base.OnPostInstall();
        }

        protected override bool OnDataButton(Bentley.DgnPlatformNET.DgnButtonEvent ev)
        {
            ReadGridLine();
            
            base.OnReinitialize();

            return true;
        }

        protected override bool OnResetButton(Bentley.DgnPlatformNET.DgnButtonEvent ev)
        {
            ExitTool();
            return true;
        }

       public static void InstallNewInstance()
        {
            GridLineRead tool = new GridLineRead(0, 0);
            tool.InstallTool();
        }
    }
    
    public sealed class CmdInterface
    {
        public static void Read(string unparsed)
        {
            GridLineTest.Instance().StartReadTool();
        }
    }
    
    [Bentley.MstnPlatformNET.AddInAttribute(MdlTaskID = "gridline")]
    public sealed class GridLineTest : Bentley.MstnPlatformNET.AddIn
    {

        private static GridLineTest s_Addin = null;

        /*---------------------------------------------------------------------------------**//**
        * Constructor.
        /*--------------+---------------+---------------+---------------+---------------+------*/
        public GridLineTest(System.IntPtr mdlDesc)
            : base(mdlDesc)
        {
            s_Addin = this;
        }

        /*---------------------------------------------------------------------------------**//**
        * Required Run method of AddIn class.
        /*--------------+---------------+---------------+---------------+---------------+------*/
        protected override int Run(string[] commandLine)
        {
            return 0;
        }

        /*---------------------------------------------------------------------------------**//**
        * Returns an instance of the Addin class.
        * @bsimethod                                                              Bentley Systems
        /*--------------+---------------+---------------+---------------+---------------+------*/
        internal static GridLineTest Instance()
        {
            return s_Addin;
        }
        
        internal void StartReadTool()
        {
            GridLineRead.InstallNewInstance();
        }
    }
}
